import java.util.ArrayList;
import java.util.List;
import java.io.IOException;

import org.jdom2.Document;
import org.jdom2.Attribute;
import org.jdom2.JDOMException;


/**
 * Class that provides input/ output info and other variables an adapter may use
 * by reading from featuremodel generated using FeatureModelConfigurator.
 */
public class Adapter extends UserSetIOReader {
	//Hard coded input/ output (because the project directory is designed like this)
	public final String in_fmName = "featuremodel.xml";
	public final String in_fmPath = "./resources/output/";
	
	private final String in_scPath = "./resources/input/sensorconfig/";
	public final String scAutoPath = in_scPath+"autogenerated/";
	public final String scOtherPath = in_scPath+"other/";
	
	//Variables used exclusively by adapters
	public final String mode;
	public final List<Document> sensorConfigList;
	public final String botName;
	
	/**
	 * Constructor defining a general adapter.
	 * 
	 * When reading input, scAutoPath, scOtherPath and in_fmPath may contain other 
	 * files that will be filtered out.
	 * 
	 * @throws IOException 
	 * @throws JDOMException 
	 */
	public Adapter() throws JDOMException, IOException {
		super();
		
		Document featureModel = XMLHelper.readXml(in_fmPath+in_fmName);
		
		//Get the selected mode (either NO_CHANGE or Custom) as String
		List<Attribute> modeList = XMLHelper.xQueryAttributes(featureModel, "//mode/@param");
		if(modeList.size()!=1) {
			System.err.println("Invalid input in \'"+in_fmName+"\': NO_CHANGE/ Custom parameter could not be parsed.");
			System.exit(-1);
		}
		
		mode = modeList.get(0).getValue();
		
		//Get the name of the selected bot as String
		List<Attribute> botList = XMLHelper.xQueryAttributes(featureModel, "//templates/@name");
		if(botList.size()!=1) {
			System.err.println("Invalid input in \'"+in_fmName+"\': No or multiple bots selected.");
			System.exit(-1);
		}
		
		botName = botList.get(0).getValue();
		
		//Get all selected sensorconfig files as List<Document>
		sensorConfigList = new ArrayList<Document>();
		
		switch(mode) {
		case "NO_CHANGE":
			break;
		case "Custom":
			List<Attribute> selectedSensorList = XMLHelper.xQueryAttributes(featureModel, "//sensor/@name");
			
			for(Attribute selectedSensor : selectedSensorList) {
				String fileName = selectedSensor.getValue()+SC_ENDING;
				
				List<String> fileMatchAutoList = XMLHelper.findFile(fileName, scAutoPath);
				List<String> fileMatchOtherList = XMLHelper.findFile(fileName, scOtherPath);
				List<String> fileMatchList = new ArrayList<String>();
				fileMatchList.addAll(fileMatchAutoList);
				fileMatchList.addAll(fileMatchOtherList);
				
				if(fileMatchList.isEmpty()) {
					System.err.println("Cannot handle \'"+fileName+"\': No input file known.");
					System.exit(-1);
				}
				if(fileMatchList.size()!=1) {
					System.err.println("Cannot handle \'"+fileName+"\': Multiple input files found:");
					for(String match : fileMatchList) {
						System.err.println("\'"+match+"\'");
					}
					System.exit(-1);
				}
				
				sensorConfigList.add(XMLHelper.readXml(fileMatchList.get(0)));
			}
			break;
		default:
			System.err.println("Invalid input in \'"+in_fmName+"\': NO_CHANGE/ Custom parameter could not be parsed.");
			System.exit(-1);
		}
	}
}
